/*
 *
 * Copyright (c) 1998 Scott Oaks. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and
 * without fee is hereby granted.
 *
 * This sample source code is provided for example only,
 * on an unsupported, as-is basis. 
 *
 * AUTHOR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. AUTHOR SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  AUTHOR
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */


import java.security.*;
import java.security.interfaces.*;
import java.security.spec.*;

public class XYZSign extends Signature implements Cloneable {
	private DSAPublicKey pub;
	private DSAPrivateKey priv;
	private MessageDigest md;

	public XYZSign() throws NoSuchAlgorithmException {
		super("XYZSign");
		md = MessageDigest.getInstance("SHA");
	}

	public void engineInitVerify(PublicKey publicKey) throws InvalidKeyException {
		try {
			pub = (DSAPublicKey) publicKey;
		} catch (ClassCastException cce) {
			throw new InvalidKeyException("Wrong public key type");
		}
	}

	public void engineInitSign(PrivateKey privateKey) throws InvalidKeyException {
		try {
			priv = (DSAPrivateKey) privateKey;
		} catch (ClassCastException cce) {
			throw new InvalidKeyException("Wrong private key type");
		}
	}

	public void engineUpdate(byte b) throws SignatureException {
		try {
			md.update(b);
		} catch (NullPointerException npe) {
			throw new SignatureException("No SHA digest found");
		}
	}

	public void engineUpdate(byte b[], int offset, int length) throws SignatureException {
		try {
			md.update(b, offset, length);
		} catch (NullPointerException npe) {
			throw new SignatureException("No SHA digest found");
		}
	}

	public byte[] engineSign() throws SignatureException {
		byte b[] = null;
		try {
			b = md.digest();
		} catch (NullPointerException npe) {
			throw new SignatureException("No SHA digest found");
		}
		return crypt(b, priv);
	}

	public boolean engineVerify(byte[] sigBytes) throws SignatureException {
		byte b[] = null;
		try {
			b = md.digest();
		} catch (NullPointerException npe) {
			throw new SignatureException("No SHA digest found");
		}
		byte sig[] = crypt(sigBytes, pub);
		return MessageDigest.isEqual(sig, b);
	}

	public void engineSetParameter(String param, Object value) {
		throw new InvalidParameterException("No parameters");
	}

	public void engineSetParameter(AlgorithmParameterSpec aps) {
		throw new InvalidParameterException("No parameters");
	}

	public Object engineGetParameter(String param) {
		throw new InvalidParameterException("No parameters");
	}

	public void engineReset() {
	}

	private byte[] crypt(byte s[], DSAKey key) {
		DSAParams p = key.getParams();
		int rotValue = p.getP().intValue();
		byte d[] = rot(s, (byte) rotValue);
		return d;
	}

	private byte[] rot(byte in[], byte rotValue) {
		byte out[] = new byte[in.length];
		for (int i = 0; i < in.length; i++) {
			out[i] = (byte) (in[i] ^ rotValue);
		}
		return out;
	}
}
